import { PropType } from "vue";
import { FONT_FORMATS, toolBar } from "./config";
interface ObjectExtends {
  [key: string]: any;
}
/**
 * @description: 图片上传文件流
 * @return {*}
 */
export interface BlobInfo {
  blob: () => Blob;
}
/**
 * @description 图片上传成功后调用
 * @param {string} e 上传成功地址
 */
export interface Success {
  (e: string): void;
}
/**
 * @description 图片上传失败后调用
 * @param {string} e 提示文本
 */
export interface Failure {
  (e: string): void;
}
/**
 * @description 文件上传
 * @param {string} filetype 文件类型
 */
export interface Meta {
  filetype: string;
}
/**
 * @description 文件信息
 * @param {string} title 文件名称
 * @param {string} text 文件地址
 */
interface File {
  title: string;
  text: string;
}
/**
 * @description 文件上传成功后调用
 * @param {string} e 上传成功后的地址
 * @param {File} t 文件信息
 */
export interface CallBack {
  (e: string, t: File): void;
}

/**
 * @description 初始化参数
 * @param { string } value 富文本默认值
 * @param { string } selector 富文本选择器
 * @param { Boolean | string } resize 调整大小
 * @param { Number | string } height 富文本高度
 * @param { Number | string } width 富文本宽度
 * @param { Number | string } minWidth 富文本最小宽度
 * @param { Number | string } minHeight 富文本最小高度
 * @param { Number } maxWidth 富文本最大宽度
 * @param { Number } maxHeight 富文本最大高度
 * @param { Boolean | undefined | String } menubar 富文本菜单栏配置，设为false则隐藏，不配置则默认显示全部菜单
 * @param { boolean | string[] } toolbar  工具栏配置，设为false则隐藏
 * @param { string } fontFormats  字体样式
 * @param { string } fontsizeFormats  字体大小
 * @param { boolean } statusbar  最下方的元素路径和字数统计那一栏是否显示
 * @param { boolean } elementPath  元素路径是否显示
 * @param { string } filePickerTypes  文件选择器类型
 * @param { string } placeholder  富文本提示文字
 * @param { string } action  上传地址
 * @param { ObjectExtends } headers  请求头
 * @param { string } name  文件上传名称
 * @param { ObjectExtends } data  请求参数
 * @param { Number } imageSize  图片文件大小
 * @param { string[] } imageAccept  图片文件格式
 * @param { Number } mediaSize  媒体文件大小
 * @param { string[] } mediaAccept  媒体文件格式
 * @param { Number } fileSize  文件大小
 * @param { string[] } fileAccept  文件格式
 * @param { ObjectExtends } externalPlugins  引入插件地址
 * @param { string } languageUrl  语言包
 * @param { string } themeUrl  主题
 * @param { string } skinUrl  皮肤
 * @param { string } iconsUrl  图标
 * @param { string[] } contentCss  内容样式
 * @param { Function } finishCallBack  初始化完成回调
 * @param { Function } imagesUploadHandler  图片上传
 * @param { Function } filePickerCallback  文件上传
 * @param { string } backFileUrlName  上传文件后，返回链接字段名
 */
export interface InitData {
  value: string;
  selector: string;
  resize: Boolean | string;
  height: Number | string;
  width: Number | string;
  minWidth: Number | string;
  minHeight: Number | string;
  maxWidth: Number;
  maxHeight: Number;
  menubar: Boolean | undefined | string;
  toolbar: [] | Boolean;
  fontFormats: string;
  fontsizeFormats: String;
  statusbar: Boolean;
  elementPath: Boolean;
  filePickerTypes: string;
  placeholder: string;
  action: string;
  headers: ObjectExtends;
  name: string;
  data: ObjectExtends;
  imageSize: Number;
  imageAccept: string[];
  mediaSize: number;
  mediaAccept: string[];
  fileSize: number;
  fileAccept: string[];
  externalPlugins: ObjectExtends;
  languageUrl: string;
  themeUrl: string;
  skinUrl: string;
  iconsUrl: string;
  contentCss: string[];
  finishCallBack: () => void;
  imagesUploadHandler: (
    blobInfo: BlobInfo,
    success: Success,
    failure: Failure
  ) => void;
  filePickerCallback: (callback: CallBack, meta: Meta) => void;
  backFileUrlName: string;
}

/**
 * @description: 请求头
 */
export type AxiosRequestHeaders = Record<string, string | number | boolean>;

/**
 * @description: 富文本编辑器工具栏配置
 */
export const editorProps = {
  value: {
    //富文本默认值
    type: String as PropType<string>,
    default: ""
  },
  selector: {
    //富文本选择器
    type: String as PropType<string>,
    default: ".tinymce"
  },
  resize: {
    // 调整大小
    type: [Boolean, String] as PropType<boolean | "both">,
    default: false
  },
  height: {
    // 富文本高度
    type: [Number, String] as PropType<number | string>,
    default: 500
  },
  width: {
    // 富文本宽度
    type: [Number, String] as PropType<number | string>,
    default: "100%"
  },
  minWidth: {
    // 富文本最小宽度
    type: [Number, String] as PropType<number | string>,
    default: 1000
  },
  minHeight: {
    // 富文本最小高度
    type: [Number, String] as PropType<number | string>,
    default: 500
  },
  maxWidth: {
    // 富文本最大宽度
    type: Number as PropType<number>,
    default: 2000
  },
  maxHeight: {
    // 富文本最大高度
    type: Number as PropType<number>,
    default: 1200
  },
  menubar: {
    // 富文本菜单栏配置，设为false则隐藏，不配置则默认显示全部菜单
    type: [Boolean, undefined, String] as PropType<
      boolean | undefined | string
    >,
    default: false
  },
  toolbar: {
    // 工具栏配置，设为false则隐藏
    type: [Array, Boolean] as PropType<string[] | boolean>,
    default: () => toolBar
  },
  fontFormats: {
    // 字体样式
    type: String as PropType<string>,
    default: () => FONT_FORMATS
  },
  fontsizeFormats: {
    // 字体大小
    type: String as PropType<string>,
    default: () => "12px 14px 16px 18px 20px 24px 28px 32px 36px"
  },
  statusbar: {
    // 最下方的元素路径和字数统计那一栏是否显示
    type: Boolean as PropType<boolean>,
    default: false
  },
  elementPath: {
    // 元素路径是否显示
    type: Boolean as PropType<boolean>,
    default: false
  },
  filePickerTypes: {
    // 文件选择器类型
    type: String as PropType<string>,
    default: "file image media"
  },
  placeholder: {
    // 富文本提示文字
    type: String as PropType<string>,
    default: "在这里输入文字"
  },
  action: {
    // 上传地址
    type: String as PropType<string>,
    required: true
  },
  headers: {
    // 请求头
    type: Object as PropType<AxiosRequestHeaders>,
    default: () => {
      return {};
    }
  },
  name: {
    // 文件上传名称
    type: String as PropType<string>,
    default: "file"
  },
  data: {
    // 请求参数
    type: Object as PropType<ObjectExtends>,
    default: () => {
      return {};
    }
  },
  imageSize: {
    // 图片文件大小
    type: Number as PropType<number>,
    default: 2
  },
  imageAccept: {
    // 图片文件格式
    type: Array as PropType<string[]>,
    default: () => {
      return [".gif", ".jpg", ".jpeg", ".png"];
    }
  },
  mediaSize: {
    // 媒体文件大小
    type: Number as PropType<number>,
    default: 10
  },
  mediaAccept: {
    // 媒体文件格式
    type: Array as PropType<string[]>,
    default: () => {
      return [".mp3", ".mp4"];
    }
  },
  fileSize: {
    // 文件大小
    type: Number as PropType<number>,
    default: 10
  },
  fileAccept: {
    // 文件格式
    type: Array as PropType<string[]>,
    default: () => {
      return [
        ".pdf",
        ".doc",
        ".docx",
        ".xls",
        ".xlsx",
        ".ppt",
        ".pptx",
        ".txt",
        ".zip",
        ".rar"
      ];
    }
  },
  externalPlugins: {
    // 引入插件地址
    type: Object as PropType<ObjectExtends>,
    default: () => {}
  },
  languageUrl: {
    // 语言包
    type: String as PropType<string>,
    default: () => {
      return new URL(`./tinymce/js/tinymce/langs/zh_CN.js`, import.meta.url)
        .href;
    }
  },
  themeUrl: {
    // 主题
    type: String as PropType<string>,
    default: () => {
      return new URL(
        `./tinymce/js/tinymce/themes/silver/theme.min.js`,
        import.meta.url
      ).href;
    }
  },
  skinUrl: {
    // 皮肤
    type: String as PropType<string>,
    default: () => {
      return new URL(
        `./tinymce/js/tinymce/skins/ui/oxide/skin.min.css`,
        import.meta.url
      ).href;
    }
  },
  iconsUrl: {
    // 图标
    type: String as PropType<string>,
    default: () => {
      return new URL(
        `./tinymce/js/tinymce/icons/default/icons.min.js`,
        import.meta.url
      ).href;
    }
  },
  contentCss: {
    // 内容样式
    type: Array as PropType<string[]>,
    default: () => {
      return [
        new URL(
          `./tinymce/js/tinymce/skins/ui/oxide/content.min.css`,
          import.meta.url
        ).href,
        new URL(
          `./tinymce/js/tinymce/skins/content/default/content.min.css`,
          import.meta.url
        ).href
      ];
    }
  },
  // 图片上传
  imagesUploadHandler: {
    type: Function as PropType<
      (blobInfo: BlobInfo, success: Success, failure: Failure) => void
    >,
    default: undefined
  },
  filePickerCallback: {
    // 文件上传
    type: Function as PropType<(callback: CallBack, meta: Meta) => void>,
    default: undefined
  },
  backFileUrlName: {
    // 上传文件后，返回链接字段名
    type: String as PropType<string>,
    default: "fileUrl"
  }
};
